# -*- coding: utf-8 -*-
#
# Copyright 2020 - Swiss Data Science Center (SDSC)
# A partnership between École Polytechnique Fédérale de Lausanne (EPFL) and
# Eidgenössische Technische Hochschule Zürich (ETHZ).
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Serializers for datasets."""

import os

from renku.core.models.json import dumps
from renku.core.models.jsonld import asjsonld

from .tabulate import tabulate


def tabular(client, datasets, *, columns=None):
    """Format datasets with a tabular output."""
    if not columns:
        columns = 'id,created,short_name,creators,tags,version'

    return tabulate(
        collection=datasets, columns=columns, columns_mapping=DATASETS_COLUMNS
    )


def jsonld(client, datasets, **kwargs):
    """Format datasets as JSON-LD."""
    data = [
        asjsonld(
            dataset,
            basedir=os.path.relpath(
                '.', start=str(dataset.__reference__.parent)
            )
        ) for dataset in datasets
    ]
    return dumps(data, indent=2)


DATASETS_FORMATS = {
    'tabular': tabular,
    'json-ld': jsonld,
}
"""Valid formatting options."""

DATASETS_COLUMNS = {
    'id': ('uid', 'id'),
    'created': ('created', None),
    'short_name': ('short_name', None),
    'creators': ('creators_csv', 'creators'),
    'creators_full': ('creators_full_csv', 'creators'),
    'tags': ('tags_csv', 'tags'),
    'version': ('version', None),
    'title': ('name', 'title'),
    'keywords': ('keywords_csv', 'keywords'),
}



# -*- coding: utf-8 -*-
#
# Copyright 2020 - Swiss Data Science Center (SDSC)
# A partnership between École Polytechnique Fédérale de Lausanne (EPFL) and
# Eidgenössische Technische Hochschule Zürich (ETHZ).
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Renku service cache project related models."""
import os
import shutil
import time
from datetime import datetime

from walrus import DateTimeField, IntegerField, Model, TextField

from renku.service.cache.base import BaseCache
from renku.service.config import CACHE_PROJECTS_PATH


class Project(Model):
    """User project object."""

    __database__ = BaseCache.model_db

    created_at = DateTimeField()

    project_id = TextField(primary_key=True, index=True)
    user_id = TextField(index=True)

    clone_depth = IntegerField()
    git_url = TextField()

    name = TextField()
    fullname = TextField()
    email = TextField()
    owner = TextField()
    token = TextField()

    @property
    def abs_path(self):
        """Full path of cached project."""
        return CACHE_PROJECTS_PATH / self.user_id / self.owner / self.name

    def exists(self):
        """Ensure a project exists on file system."""
        return self.abs_path.exists()

    def ttl_expired(self, ttl=None):
        """Check if project time to live has expired."""
        if not self.created_at:
            # If record does not contain created_at,
            # it means its an old record, and
            # we should mark it for deletion.
            return True

        ttl = ttl or int(os.getenv('RENKU_SVC_CLEANUP_TTL_PROJECTS', 1800))

        created_at = (self.created_at -
                      datetime.utcfromtimestamp(0)).total_seconds() * 1e+3

        age = ((time.time() * 1e+3) - created_at) / 1e+3
        return self.exists() and age >= ttl

    def purge(self):
        """Removes project from file system and cache."""
        shutil.rmtree(str(self.abs_path))
        self.delete()

    def is_locked(self, jobs):
        """Check if file locked by given jobs."""
        return bool(
            next((job for job in jobs if self.project_id in job.locked), False)
        )



# -*- coding: utf-8 -*-
#
# Copyright 2019-2020 - Swiss Data Science Center (SDSC)
# A partnership between École Polytechnique Fédérale de Lausanne (EPFL) and
# Eidgenössische Technische Hochschule Zürich (ETHZ).
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Renku service cache view tests."""
import io
import json
import os
import uuid

import pytest
from flaky import flaky

from renku.core.models.git import GitURL
from renku.service.config import INVALID_HEADERS_ERROR_CODE, \
    INVALID_PARAMS_ERROR_CODE

REMOTE_URL = 'https://dev.renku.ch/gitlab/contact/integration-test'
IT_GIT_ACCESS_TOKEN = os.getenv('IT_OAUTH_GIT_TOKEN')


@pytest.mark.service
def test_serve_api_spec(svc_client):
    """Check serving of service spec."""
    headers = {
        'Content-Type': 'application/json',
        'accept': 'application/json',
    }
    response = svc_client.get('/api/v1/spec', headers=headers)

    assert 0 != len(response.json.keys())
    assert 200 == response.status_code


@pytest.mark.service
def test_list_upload_files_all(svc_client):
    """Check list uploaded files view."""
    headers_user = {
        'Content-Type': 'application/json',
        'accept': 'application/json',
        'Renku-User-Id': 'user'
    }
    response = svc_client.get('/cache.files_list', headers=headers_user)

    assert {'result'} == set(response.json.keys())

    assert 0 == len(response.json['result']['files'])
    assert 200 == response.status_code


@pytest.mark.service
def test_list_upload_files_all_no_auth(svc_client):
    """Check error response on list uploaded files view."""
    headers = {
        'Content-Type': 'application/json',
        'accept': 'application/json',
    }
    response = svc_client.get(
        '/cache.files_list',
        headers=headers,
    )

    assert 200 == response.status_code

    assert {'error'} == set(response.json.keys())
    assert INVALID_HEADERS_ERROR_CODE == response.json['error']['code']


@pytest.mark.service
def test_file_upload(svc_client):
    """Check successful file upload."""
    headers_user = {'Renku-User-Id': '{0}'.format(uuid.uuid4().hex)}

    response = svc_client.post(
        '/cache.files_upload',
        data=dict(file=(io.BytesIO(b'this is a test'), 'datafile.txt'), ),
        headers=headers_user,
    )

    assert response
    assert 200 == response.status_code

    assert {'result'} == set(response.json.keys())
    assert isinstance(
        uuid.UUID(response.json['result']['files'][0]['file_id']), uuid.UUID
    )


@pytest.mark.service
def test_file_upload_override(svc_client):
    """Check successful file upload."""
    headers_user = {'Renku-User-Id': '{0}'.format(uuid.uuid4().hex)}

    response = svc_client.post(
        '/cache.files_upload',
        data=dict(file=(io.BytesIO(b'this is a test'), 'datafile.txt'), ),
        headers=headers_user,
    )

    assert response
    assert 200 == response.status_code

    assert {'result'} == set(response.json.keys())
    assert isinstance(
        uuid.UUID(response.json['result']['files'][0]['file_id']), uuid.UUID
    )
    old_file_id = response.json['result']['files'][0]['file_id']

    response = svc_client.post(
        '/cache.files_upload',
        data=dict(file=(io.BytesIO(b'this is a test'), 'datafile.txt'), ),
        headers=headers_user,
    )

    assert response
    assert 200 == response.status_code

    assert {'error'} == set(response.json.keys())
    assert INVALID_PARAMS_ERROR_CODE == response.json['error']['code']
    assert 'file exists' == response.json['error']['reason']

    response = svc_client.post(
        '/cache.files_upload',
        data=dict(file=(io.BytesIO(b'this is a test'), 'datafile.txt'), ),
        query_string={'override_existing': True},
        headers=headers_user,
    )

    assert response
    assert 200 == response.status_code

    assert {'result'} == set(response.json.keys())
    assert isinstance(
        uuid.UUID(response.json['result']['files'][0]['file_id']), uuid.UUID
    )
    assert old_file_id != response.json['result']['files'][0]['file_id']


@pytest.mark.service
def test_file_upload_same_file(svc_client):
    """Check successful file upload with same file uploaded twice."""
    headers_user1 = {'Renku-User-Id': '{0}'.format(uuid.uuid4().hex)}
    response = svc_client.post(
        '/cache.files_upload',
        data=dict(file=(io.BytesIO(b'this is a test'), 'datafile.txt'), ),
        headers=headers_user1,
    )

    assert response
    assert 200 == response.status_code

    assert {'result'} == set(response.json.keys())

    assert isinstance(
        uuid.UUID(response.json['result']['files'][0]['file_id']), uuid.UUID
    )

    response = svc_client.post(
        '/cache.files_upload',
        data=dict(file=(io.BytesIO(b'this is a test'), 'datafile.txt'), ),
        headers=headers_user1,
    )

    assert response
    assert 200 == response.status_code
    assert {'error'} == set(response.json.keys())
    assert INVALID_PARAMS_ERROR_CODE == response.json['error']['code']
    assert 'file exists' == response.json['error']['reason']


@pytest.mark.service
def test_file_upload_no_auth(svc_client):
    """Check failed file upload."""
    response = svc_client.post(
        '/cache.files_upload',
        data=dict(file=(io.BytesIO(b'this is a test'), 'datafile.txt'), ),
    )

    assert response
    assert 200 == response.status_code

    assert {'error'} == set(response.json.keys())
    assert INVALID_HEADERS_ERROR_CODE == response.json['error']['code']


@pytest.mark.service
def test_file_upload_with_users(svc_client):
    """Check successful file upload and listing based on user auth header."""
    headers_user1 = {'Renku-User-Id': '{0}'.format(uuid.uuid4().hex)}
    headers_user2 = {'Renku-User-Id': '{0}'.format(uuid.uuid4().hex)}

    response = svc_client.post(
        '/cache.files_upload',
        data=dict(file=(io.BytesIO(b'this is a test'), 'datafile1.txt'), ),
        headers=headers_user1
    )

    assert {'result'} == set(response.json.keys())

    file_id = response.json['result']['files'][0]['file_id']
    assert file_id
    assert 200 == response.status_code

    response = svc_client.post(
        '/cache.files_upload',
        data=dict(file=(io.BytesIO(b'this is a test'), 'datafile1.txt'), ),
        headers=headers_user2
    )

    assert response
    assert {'result'} == set(response.json.keys())

    response = svc_client.get('/cache.files_list', headers=headers_user1)

    assert response

    assert {'result'} == set(response.json.keys())
    assert 1 == len(response.json['result']['files'])

    file = response.json['result']['files'][0]
    assert file_id == file['file_id']
    assert 0 < file['file_size']


@pytest.mark.service
@pytest.mark.integration
@flaky(max_runs=10, min_passes=1)
def test_clone_projects_no_auth(svc_client):
    """Check error on cloning of remote repository."""
    payload = {
        'git_url': REMOTE_URL,
    }

    response = svc_client.post(
        '/cache.project_clone', data=json.dumps(payload)
    )

    assert {'error'} == set(response.json.keys())
    assert INVALID_HEADERS_ERROR_CODE == response.json['error']['code']

    err_message = 'user identification is incorrect or missing'
    assert err_message == response.json['error']['reason']

    headers = {
        'Content-Type': 'application/json',
        'accept': 'application/json',
        'Renku-User-Id': '{0}'.format(uuid.uuid4().hex),
        'Renku-User-FullName': 'Just Sam',
        'Renku-User-Email': 'contact@justsam.io',
        'Authorization': 'Bearer notatoken',
    }

    response = svc_client.post(
        '/cache.project_clone', data=json.dumps(payload), headers=headers
    )

    assert response
    assert {'result'} == set(response.json.keys())


@pytest.mark.service
@pytest.mark.integration
@flaky(max_runs=10, min_passes=1)
def test_clone_projects_with_auth(svc_client):
    """Check cloning of remote repository."""
    headers = {
        'Content-Type': 'application/json',
        'accept': 'application/json',
        'Renku-User-Id': '{0}'.format(uuid.uuid4().hex),
        'Renku-User-FullName': 'Just Sam',
        'Renku-User-Email': 'contact@justsam.io',
        'Authorization': 'Bearer {0}'.format(IT_GIT_ACCESS_TOKEN),
    }

    payload = {
        'git_url': REMOTE_URL,
    }

    response = svc_client.post(
        '/cache.project_clone', data=json.dumps(payload), headers=headers
    )

    assert response
    assert {'result'} == set(response.json.keys())


@pytest.mark.service
@pytest.mark.integration
@flaky(max_runs=10, min_passes=1)
def test_clone_projects_multiple(svc_client):
    """Check multiple cloning of remote repository."""
    project_ids = []

    headers = {
        'Content-Type': 'application/json',
        'Renku-User-Id': '{0}'.format(uuid.uuid4().hex),
        'Renku-User-FullName': 'Just Sam',
        'Renku-User-Email': 'contact@justsam.io',
        'Authorization': 'Bearer {0}'.format(IT_GIT_ACCESS_TOKEN),
    }

    payload = {
        'git_url': REMOTE_URL,
    }

    response = svc_client.post(
        '/cache.project_clone', data=json.dumps(payload), headers=headers
    )
    assert response

    assert {'result'} == set(response.json.keys())
    project_ids.append(response.json['result'])

    response = svc_client.post(
        '/cache.project_clone', data=json.dumps(payload), headers=headers
    )

    assert response
    assert {'result'} == set(response.json.keys())
    project_ids.append(response.json['result'])

    response = svc_client.post(
        '/cache.project_clone', data=json.dumps(payload), headers=headers
    )

    assert response
    assert {'result'} == set(response.json.keys())
    project_ids.append(response.json['result'])

    response = svc_client.post(
        '/cache.project_clone', data=json.dumps(payload), headers=headers
    )

    assert response
    assert {'result'} == set(response.json.keys())
    last_pid = response.json['result']['project_id']

    response = svc_client.get('/cache.project_list', headers=headers)

    assert response
    assert {'result'} == set(response.json.keys())

    pids = [p['project_id'] for p in response.json['result']['projects']]
    assert last_pid in pids

    for inserted in project_ids:
        assert inserted['project_id'] not in pids


@pytest.mark.service
@pytest.mark.integration
@flaky(max_runs=10, min_passes=1)
def test_clone_projects_list_view_errors(svc_client):
    """Check cache state of cloned projects with no headers."""
    headers = {
        'Content-Type': 'application/json',
        'Renku-User-Id': '{0}'.format(uuid.uuid4().hex),
        'Renku-User-FullName': 'Just Sam',
        'Renku-User-Email': 'contact@justsam.io',
        'Authorization': 'Bearer {0}'.format(IT_GIT_ACCESS_TOKEN),
    }

    payload = {
        'git_url': REMOTE_URL,
    }

    response = svc_client.post(
        '/cache.project_clone', data=json.dumps(payload), headers=headers
    )
    assert response
    assert {'result'} == set(response.json.keys())

    assert isinstance(
        uuid.UUID(response.json['result']['project_id']), uuid.UUID
    )

    response = svc_client.get(
        '/cache.project_list',
        # no auth headers, expected error
    )
    assert response

    assert {'error'} == set(response.json.keys())
    assert INVALID_HEADERS_ERROR_CODE == response.json['error']['code']

    response = svc_client.get('/cache.project_list', headers=headers)

    assert response
    assert {'result'} == set(response.json.keys())
    assert 1 == len(response.json['result']['projects'])

    project = response.json['result']['projects'][0]
    assert isinstance(uuid.UUID(project['project_id']), uuid.UUID)
    assert isinstance(GitURL.parse(project['git_url']), GitURL)


@pytest.mark.service
@pytest.mark.integration
@flaky(max_runs=10, min_passes=1)
def test_clone_projects_invalid_headers(svc_client):
    """Check cache state of cloned projects with invalid headers."""
    headers = {
        'Content-Type': 'application/json',
        'accept': 'application/json',
        'Renku-User-Id': '{0}'.format(uuid.uuid4().hex),
        'Renku-User-FullName': 'Not Sam',
        'Renku-User-Email': 'not@sam.io',
        'Authorization': 'Bearer not-a-token',
    }

    payload = {
        'git_url': REMOTE_URL,
    }

    response = svc_client.post(
        '/cache.project_clone',
        data=json.dumps(payload),
        headers=headers,
    )
    assert response
    assert {'result'} == set(response.json.keys())

    response = svc_client.get(
        '/cache.project_list',
        # no auth headers, expected error
    )

    assert response
    assert {'error'} == set(response.json.keys())
    assert INVALID_HEADERS_ERROR_CODE == response.json['error']['code']

    response = svc_client.get('/cache.project_list', headers=headers)
    assert response
    assert {'result'} == set(response.json.keys())
    assert 1 == len(response.json['result']['projects'])


@pytest.mark.service
def test_upload_zip_unpack_archive(datapack_zip, svc_client_with_repo):
    """Upload zip archive with unpack."""
    svc_client, headers, project_id, _ = svc_client_with_repo
    headers.pop('Content-Type')

    response = svc_client.post(
        '/cache.files_upload',
        data=dict(
            file=(io.BytesIO(datapack_zip.read_bytes()), datapack_zip.name),
        ),
        query_string={
            'unpack_archive': True,
            'override_existing': True,
        },
        headers=headers
    )

    assert response

    assert 200 == response.status_code
    assert {'result'} == set(response.json.keys())
    assert response.json['result']['files']

    for file_ in response.json['result']['files']:
        assert not file_['is_archive']
        assert not file_['unpack_archive']


@pytest.mark.service
def test_upload_zip_archive(datapack_zip, svc_client_with_repo):
    """Upload zip archive."""
    svc_client, headers, project_id, _ = svc_client_with_repo
    headers.pop('Content-Type')

    response = svc_client.post(
        '/cache.files_upload',
        data=dict(
            file=(io.BytesIO(datapack_zip.read_bytes()), datapack_zip.name),
        ),
        query_string={
            'unpack_archive': False,
            'override_existing': True,
        },
        headers=headers
    )

    assert response

    assert 200 == response.status_code
    assert {'result'} == set(response.json.keys())
    assert 1 == len(response.json['result']['files'])

    for file_ in response.json['result']['files']:
        assert file_['is_archive']
        assert not file_['unpack_archive']


@pytest.mark.service
def test_upload_tar_unpack_archive(datapack_tar, svc_client_with_repo):
    """Upload zip archive with unpack."""
    svc_client, headers, project_id, _ = svc_client_with_repo
    headers.pop('Content-Type')

    response = svc_client.post(
        '/cache.files_upload',
        data=dict(
            file=(io.BytesIO(datapack_tar.read_bytes()), datapack_tar.name),
        ),
        query_string={
            'unpack_archive': True,
            'override_existing': True,
        },
        headers=headers
    )

    assert response

    assert 200 == response.status_code
    assert {'result'} == set(response.json.keys())
    assert response.json['result']['files']

    for file_ in response.json['result']['files']:
        assert not file_['is_archive']
        assert not file_['unpack_archive']

    response = svc_client.get(
        '/cache.files_list',
        headers=headers,
    )

    assert response
    assert 200 == response.status_code
    assert {'result'} == set(response.json.keys())
    assert response.json['result']['files']

    dirs = filter(lambda x: x['is_dir'], response.json['result']['files'])
    assert list(dirs)

    files = filter(lambda x: not x['is_dir'], response.json['result']['files'])
    assert list(files)

    paths = [_file['relative_path'] for _file in files]
    assert sorted(paths) == paths


@pytest.mark.service
def test_upload_tar_archive(datapack_tar, svc_client_with_repo):
    """Upload zip archive."""
    svc_client, headers, project_id, _ = svc_client_with_repo
    headers.pop('Content-Type')

    response = svc_client.post(
        '/cache.files_upload',
        data=dict(
            file=(io.BytesIO(datapack_tar.read_bytes()), datapack_tar.name),
        ),
        query_string={
            'unpack_archive': False,
            'override_existing': True,
        },
        headers=headers
    )

    assert response

    assert 200 == response.status_code
    assert {'result'} == set(response.json.keys())
    assert 1 == len(response.json['result']['files'])

    for file_ in response.json['result']['files']:
        assert file_['is_archive']
        assert not file_['unpack_archive']


@pytest.mark.service
def test_field_upload_resp_fields(datapack_tar, svc_client_with_repo):
    """Check response fields."""
    svc_client, headers, project_id, _ = svc_client_with_repo
    headers.pop('Content-Type')

    response = svc_client.post(
        '/cache.files_upload',
        data=dict(
            file=(io.BytesIO(datapack_tar.read_bytes()), datapack_tar.name),
        ),
        query_string={
            'unpack_archive': True,
            'override_existing': True,
        },
        headers=headers
    )

    assert response

    assert 200 == response.status_code

    assert {'result'} == set(response.json.keys())
    assert response.json['result']['files']

    assert {
        'content_type',
        'file_id',
        'file_name',
        'file_size',
        'is_archive',
        'created_at',
        'is_dir',
        'unpack_archive',
        'relative_path',
    } == set(response.json['result']['files'][0].keys())

    assert not response.json['result']['files'][0]['is_archive']
    assert not response.json['result']['files'][0]['unpack_archive']

    rel_path = response.json['result']['files'][0]['relative_path']
    assert rel_path.startswith(datapack_tar.name) and 'unpacked' in rel_path



# -*- coding: utf-8 -*-
#
# Copyright 2020 - Swiss Data Science Center (SDSC)
# A partnership between École Polytechnique Fédérale de Lausanne (EPFL) and
# Eidgenössische Technische Hochschule Zürich (ETHZ).
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Renku service jobs management."""
from renku.service.cache.base import BaseCache
from renku.service.cache.models.job import Job
from renku.service.cache.serializers.job import JobSchema


class JobManagementCache(BaseCache):
    """Job management cache."""

    job_schema = JobSchema()

    def make_job(self, user, job_data=None, locked=None):
        """Cache job state under user hash set."""
        if isinstance(locked, str) or isinstance(locked, int):
            locked = {locked}

        if job_data:
            job_data.update({'user_id': user.user_id})
        else:
            job_data = {'user_id': user.user_id}

        job_obj = self.job_schema.load(job_data)
        job_obj.save()

        for lock_obj in locked or []:
            job_obj.locked.add(lock_obj)

        return job_obj

    @staticmethod
    def get_job(user, job_id):
        """Get user job."""
        try:
            job_obj = Job.get((Job.job_id == job_id) &
                              (Job.user_id == user.user_id))
        except ValueError:
            return

        return job_obj

    @staticmethod
    def get_jobs(user):
        """Get all user jobs."""
        return Job.query((Job.user_id == user.user_id))

    @staticmethod
    def invalidate_job(user, job_id):
        """Remove users job record."""
        job_obj = JobManagementCache.get_job(user, job_id)

        if job_obj:
            job_obj.delete()

        return job_obj



# -*- coding: utf-8 -*-
#
# Copyright 2018-2020- Swiss Data Science Center (SDSC)
# A partnership between École Polytechnique Fédérale de Lausanne (EPFL) and
# Eidgenössische Technische Hochschule Zürich (ETHZ).
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Represent provenance processes."""

import weakref

from renku.core.models import jsonld as jsonld
from renku.core.models.entities import CommitMixin


@jsonld.s(
    type=[
        'wfdesc:Process',
        'prov:Entity',
        'prov:Plan',
    ],
    context={
        'wfdesc': 'http://purl.org/wf4ever/wfdesc#',
        'prov': 'http://www.w3.org/ns/prov#',
    },
    cmp=False,
)
class Process(CommitMixin):
    """Represent a process."""

    _activity = jsonld.ib(
        context='prov:activity',
        kw_only=True,
        converter=weakref.ref,
        type='renku.core.models.provenance.activities.Activity'
    )

    @property
    def activity(self):
        """Return the activity object."""
        return self._activity()


@jsonld.s(
    type=[
        'wfdesc:Workflow',
        'prov:Entity',
        'prov:Plan',
    ],
    context={
        'wfdesc': 'http://purl.org/wf4ever/wfdesc#',
        'prov': 'http://www.w3.org/ns/prov#',
    },
    cmp=False,
)
class Workflow(Process):
    """Represent workflow with subprocesses."""

    subprocesses = jsonld.ib(context='wfdesc:hasSubProcess', kw_only=True)

    @subprocesses.default
    def default_subprocesses(self):
        """Load subprocesses."""
        return [
            subprocess.association.plan
            for subprocess in self.activity.subprocesses.values()
        ]



# -*- coding: utf-8 -*-
#
# Copyright 2020 - Swiss Data Science Center (SDSC)
# A partnership between École Polytechnique Fédérale de Lausanne (EPFL) and
# Eidgenössische Technische Hochschule Zürich (ETHZ).
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Renku service config."""
import os
import tempfile
from pathlib import Path

GIT_ACCESS_DENIED_ERROR_CODE = -32000
GIT_UNKNOWN_ERROR_CODE = -32001

RENKU_EXCEPTION_ERROR_CODE = -32100
REDIS_EXCEPTION_ERROR_CODE = -32200

INVALID_HEADERS_ERROR_CODE = -32601
INVALID_PARAMS_ERROR_CODE = -32602
INTERNAL_FAILURE_ERROR_CODE = -32603

SERVICE_NAME = 'Renku Service'
OPENAPI_VERSION = '2.0'
API_VERSION = 'v1'

SWAGGER_URL = '/api/docs'
API_SPEC_URL = os.getenv(
    'RENKU_SVC_SWAGGER_URL', '/api/{0}/spec'.format(API_VERSION)
)

PROJECT_CLONE_DEPTH_DEFAULT = int(os.getenv('PROJECT_CLONE_DEPTH_DEFAULT', 1))

CACHE_DIR = os.getenv('CACHE_DIR', tempfile.TemporaryDirectory().name)
CACHE_UPLOADS_PATH = Path(CACHE_DIR) / Path('uploads')
CACHE_UPLOADS_PATH.mkdir(parents=True, exist_ok=True)

CACHE_PROJECTS_PATH = Path(CACHE_DIR) / Path('projects')
CACHE_PROJECTS_PATH.mkdir(parents=True, exist_ok=True)

TAR_ARCHIVE_CONTENT_TYPE = 'application/x-tar'
ZIP_ARCHIVE_CONTENT_TYPE = 'application/zip'

SUPPORTED_ARCHIVES = [
    TAR_ARCHIVE_CONTENT_TYPE,
    ZIP_ARCHIVE_CONTENT_TYPE,
]

SERVICE_PREFIX = os.getenv('CORE_SERVICE_PREFIX', '/')
LOGGER_CONFIG_FILE = Path('renku') / 'service' / 'logging.yaml'



# -*- coding: utf-8 -*-
#
# Copyright 2017-2020 - Swiss Data Science Center (SDSC)
# A partnership between École Polytechnique Fédérale de Lausanne (EPFL) and
# Eidgenössische Technische Hochschule Zürich (ETHZ).
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Dataset tests."""
import os
import shutil
import stat
from pathlib import Path

import pytest
from git import Repo
from tests.utils import raises

from renku.core import errors
from renku.core.commands.dataset import add_file, create_dataset, \
    list_datasets, list_files
from renku.core.models.datasets import Dataset, DatasetFile
from renku.core.models.provenance.agents import Person


@pytest.mark.parametrize(
    'scheme, path, overwrite, error',
    [('', 'temp', False, None), ('file://', 'temp', True, None),
     ('', 'tempp', False, errors.ParameterError),
     ('http://', 'example.com/file', False, None),
     ('https://', 'example.com/file', True, None),
     ('bla://', 'file', False, errors.UrlSchemeNotSupported)]
)
def test_data_add(
    scheme, path, overwrite, error, client, data_file, directory_tree,
    dataset_responses
):
    """Test data import."""
    with raises(error):
        if path == 'temp':
            path = str(data_file)
        elif path == 'tempdir':
            path = str(directory_tree)

        with client.with_dataset('dataset', create=True) as d:
            d.creator = [{
                'name': 'me',
                'email': 'me@example.com',
                'identifier': 'me_id'
            }]

            client.add_data_to_dataset(
                d, ['{}{}'.format(scheme, path)], overwrite=overwrite
            )

        with open('data/dataset/file') as f:
            assert f.read() == '1234'

        assert d.find_file('data/dataset/file')

        # check that the imported file is read-only
        assert not os.access(
            'data/dataset/file', stat.S_IXUSR | stat.S_IXGRP | stat.S_IXOTH
        )

        # check the linking
        if scheme in ('', 'file://'):
            shutil.rmtree('./data/dataset')
            with client.with_dataset('dataset') as d:
                d.creator = [{
                    'name': 'me',
                    'email': 'me@example.com',
                    'identifier': 'me_id'
                }]
                client.add_data_to_dataset(
                    d, ['{}{}'.format(scheme, path)], overwrite=True
                )
            assert os.path.exists('data/dataset/file')


def test_data_add_recursive(directory_tree, client):
    """Test recursive data imports."""
    with client.with_dataset('dataset', create=True) as dataset:
        dataset.creator = [{
            'name': 'me',
            'email': 'me@example.com',
            'identifier': 'me_id'
        }]
        client.add_data_to_dataset(
            dataset, [directory_tree.join('dir2').strpath]
        )

        assert os.path.basename(
            os.path.dirname(dataset.files[0].path)
        ) == 'dir2'


def test_git_repo_import(client, dataset, tmpdir, data_repository):
    """Test an import from a git repository."""
    # add data from local repo
    client.add_data_to_dataset(
        dataset,
        [os.path.join(os.path.dirname(data_repository.git_dir), 'dir2')]
    )
    assert os.stat('data/dataset/dir2/file2')
    assert dataset.files[0].path.endswith('dir2/file2')


@pytest.mark.parametrize(
    'creators', [
        [Person(name='me', email='me@example.com')],
        [{
            'name': 'me',
            'email': 'me@example.com',
        }],
    ]
)
def test_creator_parse(creators, data_file):
    """Test that different options for specifying creators work."""
    f = DatasetFile(path='file', creator=creators)
    creator = Person(name='me', email='me@example.com')
    assert creator in f.creator

    # email check
    with pytest.raises(ValueError):
        Person(name='me', email='meexample.com')

    # creators must be a set or list of dicts or Person
    with pytest.raises(ValueError):
        f = DatasetFile(path='file', creator=['name'])


def test_dataset_serialization(dataset):
    """Test dataset (de)serialization."""
    dataset_metadata = dataset.asjsonld()
    dataset = Dataset.from_jsonld(dataset_metadata)

    # assert that all attributes found in metadata are set in the instance
    assert dataset.created
    assert dataset.creator
    assert dataset.identifier
    assert dataset.name
    assert dataset.path
    assert dataset._project

    # check values
    assert str(dataset.created.isoformat()) == dataset_metadata.get('created')
    assert dataset.creator[0].email == dataset_metadata.get('creator'
                                                            )[0].get('email')
    assert dataset.identifier == dataset_metadata.get('identifier')
    assert dataset.name == dataset_metadata.get('name')
    assert dataset.path == dataset_metadata.get('path')


def test_create_dataset_custom_message(project):
    """Test create dataset custom message."""
    create_dataset(
        'ds1',
        title='',
        description='',
        creators=[],
        commit_message='my awesome dataset'
    )

    last_commit = Repo('.').head.commit
    assert 'my awesome dataset' == last_commit.message


def test_list_datasets_default(project):
    """Test a default dataset listing."""
    create_dataset(
        'ds1',
        title='',
        description='',
        creators=[],
        commit_message='my awesome dataset'
    )

    datasets = list_datasets()

    assert isinstance(datasets, list)
    assert 'ds1' in [ds.name for ds in datasets]


def test_list_files_default(project, tmpdir):
    """Test a default file listing."""
    create_dataset(
        'ds1',
        title='',
        description='',
        creators=[],
        commit_message='my awesome dataset'
    )
    data_file = tmpdir / Path('somefile')
    data_file.write_text('1,2,3', encoding='utf-8')

    add_file([str(data_file)], 'ds1')
    files = list_files(datasets=['ds1'])

    assert isinstance(files, list)
    assert 'somefile' in [ds.name for ds in files]



# -*- coding: utf-8 -*-
#
# Copyright 2019-2020 - Swiss Data Science Center (SDSC)
# A partnership between École Polytechnique Fédérale de Lausanne (EPFL) and
# Eidgenössische Technische Hochschule Zürich (ETHZ).
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Renku service dataset view tests."""
import io
import json
import os
import re
import shutil
import uuid
from pathlib import Path

import pytest
from flaky import flaky
from tests.utils import make_dataset_add_payload

from renku.service.config import INVALID_HEADERS_ERROR_CODE, \
    INVALID_PARAMS_ERROR_CODE, RENKU_EXCEPTION_ERROR_CODE


def assert_rpc_response(response, with_key='result'):
    """Check rpc result in response."""
    assert response and 200 == response.status_code

    response_text = re.sub(
        r'http\S+',
        '',
        json.dumps(response.json),
    )
    assert with_key in response_text


@pytest.mark.service
@pytest.mark.integration
@flaky(max_runs=30, min_passes=1)
def test_create_dataset_view(svc_client_with_repo):
    """Create a new dataset successfully."""
    svc_client, headers, project_id, _ = svc_client_with_repo

    payload = {
        'project_id': project_id,
        'short_name': '{0}'.format(uuid.uuid4().hex),
    }

    response = svc_client.post(
        '/datasets.create',
        data=json.dumps(payload),
        headers=headers,
    )

    assert response
    assert_rpc_response(response)

    assert {'short_name'} == set(response.json['result'].keys())
    assert payload['short_name'] == response.json['result']['short_name']


@pytest.mark.service
@pytest.mark.integration
@flaky(max_runs=30, min_passes=1)
def test_create_dataset_with_metadata(svc_client_with_repo):
    """Create a new dataset with metadata."""
    svc_client, headers, project_id, _ = svc_client_with_repo

    payload = {
        'project_id': project_id,
        'short_name': '{0}'.format(uuid.uuid4().hex),
        'title': 'my little dataset',
        'creators': [{
            'name': 'name123',
            'email': 'name123@ethz.ch',
            'affiliation': 'ethz'
        }],
        'description': 'my little description',
    }

    response = svc_client.post(
        '/datasets.create',
        data=json.dumps(payload),
        headers=headers,
    )

    assert response
    assert_rpc_response(response)

    assert {'short_name'} == set(response.json['result'].keys())
    assert payload['short_name'] == response.json['result']['short_name']

    params = {
        'project_id': project_id,
    }
    response = svc_client.get(
        '/datasets.list',
        query_string=params,
        headers=headers,
    )

    assert response
    assert_rpc_response(response)
    ds = next(
        ds for ds in response.json['result']['datasets']
        if ds['short_name'] == payload['short_name']
    )

    assert payload['title'] == ds['title']
    assert payload['short_name'] == ds['short_name']
    assert payload['description'] == ds['description']
    assert payload['creators'] == ds['creators']


@pytest.mark.service
@pytest.mark.integration
@flaky(max_runs=30, min_passes=1)
def test_create_dataset_invalid_creator(svc_client_with_repo):
    """Create a new dataset with metadata."""
    svc_client, headers, project_id, _ = svc_client_with_repo

    payload = {
        'project_id': project_id,
        'short_name': '{0}'.format(uuid.uuid4().hex),
        'title': 'my little dataset',
        'creators': [{
            'name': None,
            'email': 'name123@ethz.ch',
            'affiliation': 'ethz'
        }],
        'description': 'my little description',
    }

    response = svc_client.post(
        '/datasets.create',
        data=json.dumps(payload),
        headers=headers,
    )

    assert response
    assert INVALID_PARAMS_ERROR_CODE == response.json['error']['code']

    expected_err = {'creators': {'0': {'name': ['Field may not be null.']}}}
    assert expected_err == response.json['error']['reason']


@pytest.mark.service
@pytest.mark.integration
@flaky(max_runs=10, min_passes=1)
def test_create_dataset_commit_msg(svc_client_with_repo):
    """Create a new dataset successfully with custom commit message."""
    svc_client, headers, project_id, _ = svc_client_with_repo

    payload = {
        'project_id': project_id,
        'short_name': '{0}'.format(uuid.uuid4().hex),
        'commit_message': 'my awesome dataset'
    }

    response = svc_client.post(
        '/datasets.create',
        data=json.dumps(payload),
        headers=headers,
    )

    assert response
    assert_rpc_response(response)

    assert {'short_name'} == set(response.json['result'].keys())
    assert payload['short_name'] == response.json['result']['short_name']


@pytest.mark.service
@pytest.mark.integration
@flaky(max_runs=30, min_passes=1)
def test_create_dataset_view_dataset_exists(svc_client_with_repo):
    """Create a new dataset which already exists."""
    svc_client, headers, project_id, _ = svc_client_with_repo

    payload = {
        'project_id': project_id,
        'short_name': 'mydataset',
    }

    response = svc_client.post(
        '/datasets.create',
        data=json.dumps(payload),
        headers=headers,
    )

    assert response
    assert_rpc_response(response, with_key='error')

    assert RENKU_EXCEPTION_ERROR_CODE == response.json['error']['code']
    assert 'Dataset exists' in response.json['error']['reason']


@pytest.mark.service
@pytest.mark.integration
@flaky(max_runs=30, min_passes=1)
def test_create_dataset_view_unknown_param(svc_client_with_repo):
    """Create new dataset by specifying unknown parameters."""
    svc_client, headers, project_id, _ = svc_client_with_repo

    payload = {
        'project_id': project_id,
        'short_name': 'mydata',
        'remote_name': 'origin'
    }

    response = svc_client.post(
        '/datasets.create',
        data=json.dumps(payload),
        headers=headers,
    )

    assert response
    assert_rpc_response(response, with_key='error')

    assert INVALID_PARAMS_ERROR_CODE == response.json['error']['code']
    assert {'remote_name'} == set(response.json['error']['reason'].keys())


@pytest.mark.service
@pytest.mark.integration
@flaky(max_runs=30, min_passes=1)
def test_create_dataset_with_no_identity(svc_client_with_repo):
    """Create a new dataset with no identification provided."""
    svc_client, headers, project_id, _ = svc_client_with_repo

    payload = {
        'project_id': project_id,
        'short_name': 'mydata',
        'remote_name': 'origin',
    }

    response = svc_client.post(
        '/datasets.create',
        data=json.dumps(payload),
        headers={'Content-Type': headers['Content-Type']}
        # no user identity, expect error
    )

    assert response
    assert_rpc_response(response, with_key='error')

    assert INVALID_HEADERS_ERROR_CODE == response.json['error']['code']

    err_message = 'user identification is incorrect or missing'
    assert err_message == response.json['error']['reason']


@pytest.mark.service
@pytest.mark.integration
@flaky(max_runs=30, min_passes=1)
def test_add_file_view_with_no_identity(svc_client_with_repo):
    """Check identity error raise in dataset add."""
    svc_client, headers, project_id, _ = svc_client_with_repo
    payload = {
        'project_id': project_id,
        'short_name': 'mydata',
        'remote_name': 'origin',
    }

    response = svc_client.post(
        '/datasets.add',
        data=json.dumps(payload),
        headers={'Content-Type': headers['Content-Type']}
        # no user identity, expect error
    )
    assert response
    assert_rpc_response(response, with_key='error')

    assert INVALID_HEADERS_ERROR_CODE == response.json['error']['code']

    err_message = 'user identification is incorrect or missing'
    assert err_message == response.json['error']['reason']


@pytest.mark.service
@pytest.mark.integration
@flaky(max_runs=30, min_passes=1)
def test_add_file_view(svc_client_with_repo):
    """Check adding of uploaded file to dataset."""
    svc_client, headers, project_id, _ = svc_client_with_repo
    content_type = headers.pop('Content-Type')

    response = svc_client.post(
        '/cache.files_upload',
        data=dict(file=(io.BytesIO(b'this is a test'), 'datafile1.txt'), ),
        query_string={'override_existing': True},
        headers=headers
    )

    assert response
    assert 200 == response.status_code
    assert_rpc_response(response)

    assert 1 == len(response.json['result']['files'])

    file_id = response.json['result']['files'][0]['file_id']
    assert isinstance(uuid.UUID(file_id), uuid.UUID)

    payload = {
        'project_id': project_id,
        'short_name': '{0}'.format(uuid.uuid4().hex),
        'create_dataset': True,
        'files': [{
            'file_id': file_id,
        }, ]
    }
    headers['Content-Type'] = content_type

    response = svc_client.post(
        '/datasets.add',
        data=json.dumps(payload),
        headers=headers,
    )

    assert response
    assert_rpc_response(response)

    assert {'short_name', 'project_id',
            'files'} == set(response.json['result'].keys())

    assert 1 == len(response.json['result']['files'])
    assert file_id == response.json['result']['files'][0]['file_id']


@pytest.mark.service
@pytest.mark.integration
@flaky(max_runs=30, min_passes=1)
def test_add_file_commit_msg(svc_client_with_repo):
    """Check adding of uploaded file to dataset with custom commit message."""
    svc_client, headers, project_id, _ = svc_client_with_repo
    content_type = headers.pop('Content-Type')

    response = svc_client.post(
        '/cache.files_upload',
        data=dict(file=(io.BytesIO(b'this is a test'), 'datafile1.txt'), ),
        query_string={'override_existing': True},
        headers=headers
    )

    file_id = response.json['result']['files'][0]['file_id']
    assert isinstance(uuid.UUID(file_id), uuid.UUID)

    payload = {
        'commit_message': 'my awesome data file',
        'project_id': project_id,
        'short_name': '{0}'.format(uuid.uuid4().hex),
        'create_dataset': True,
        'files': [{
            'file_id': file_id,
        }, ]
    }
    headers['Content-Type'] = content_type
    response = svc_client.post(
        '/datasets.add',
        data=json.dumps(payload),
        headers=headers,
    )

    assert response
    assert_rpc_response(response)

    assert {'short_name', 'project_id',
            'files'} == set(response.json['result'].keys())

    assert 1 == len(response.json['result']['files'])
    assert file_id == response.json['result']['files'][0]['file_id']


@pytest.mark.service
@pytest.mark.integration
@flaky(max_runs=30, min_passes=1)
def test_add_file_failure(svc_client_with_repo):
    """Check adding of uploaded file to dataset with non-existing file."""
    svc_client, headers, project_id, _ = svc_client_with_repo
    content_type = headers.pop('Content-Type')

    response = svc_client.post(
        '/cache.files_upload',
        data=dict(file=(io.BytesIO(b'this is a test'), 'datafile1.txt'), ),
        query_string={'override_existing': True},
        headers=headers
    )

    file_id = response.json['result']['files'][0]['file_id']
    assert isinstance(uuid.UUID(file_id), uuid.UUID)

    payload = {
        'commit_message': 'my awesome data file',
        'project_id': project_id,
        'short_name': '{0}'.format(uuid.uuid4().hex),
        'create_dataset': True,
        'files': [{
            'file_id': file_id,
        }, {
            'file_path': 'my problem right here'
        }]
    }
    headers['Content-Type'] = content_type
    response = svc_client.post(
        '/datasets.add',
        data=json.dumps(payload),
        headers=headers,
    )

    assert response
    assert_rpc_response(response, with_key='error')

    assert {'code', 'reason'} == set(response.json['error'].keys())
    assert 'invalid file reference' in response.json['error']['reason']


@pytest.mark.service
@pytest.mark.integration
@flaky(max_runs=30, min_passes=1)
def test_list_datasets_view(svc_client_with_repo):
    """Check listing of existing datasets."""
    svc_client, headers, project_id, _ = svc_client_with_repo

    params = {
        'project_id': project_id,
    }

    response = svc_client.get(
        '/datasets.list',
        query_string=params,
        headers=headers,
    )

    assert response
    assert_rpc_response(response)

    assert {'datasets'} == set(response.json['result'].keys())
    assert 0 != len(response.json['result']['datasets'])

    assert {
        'version', 'description', 'created_at', 'short_name', 'title',
        'creators'
    } == set(response.json['result']['datasets'][0].keys())


@pytest.mark.service
@pytest.mark.integration
@flaky(max_runs=30, min_passes=1)
def test_list_datasets_view_no_auth(svc_client_with_repo):
    """Check listing of existing datasets with no auth."""
    svc_client, headers, project_id, _ = svc_client_with_repo

    params = {
        'project_id': project_id,
    }

    response = svc_client.get(
        '/datasets.list',
        query_string=params,
    )

    assert response
    assert_rpc_response(response, with_key='error')


@pytest.mark.service
@pytest.mark.integration
@flaky(max_runs=1, min_passes=1)
def test_create_and_list_datasets_view(svc_client_with_repo):
    """Create and list created dataset."""
    svc_client, headers, project_id, _ = svc_client_with_repo

    payload = {
        'project_id': project_id,
        'short_name': '{0}'.format(uuid.uuid4().hex),
    }

    response = svc_client.post(
        '/datasets.create',
        data=json.dumps(payload),
        headers=headers,
    )

    assert response

    assert_rpc_response(response)
    assert {'short_name'} == set(response.json['result'].keys())
    assert payload['short_name'] == response.json['result']['short_name']

    params_list = {
        'project_id': project_id,
    }

    response = svc_client.get(
        '/datasets.list',
        query_string=params_list,
        headers=headers,
    )

    assert response
    assert_rpc_response(response)

    assert {'datasets'} == set(response.json['result'].keys())
    assert 0 != len(response.json['result']['datasets'])
    assert {
        'creators', 'short_name', 'version', 'title', 'description',
        'created_at'
    } == set(response.json['result']['datasets'][0].keys())

    assert payload['short_name'] in [
        ds['short_name'] for ds in response.json['result']['datasets']
    ]


@pytest.mark.service
@pytest.mark.integration
@flaky(max_runs=30, min_passes=1)
def test_list_dataset_files(svc_client_with_repo):
    """Check listing of dataset files"""
    svc_client, headers, project_id, _ = svc_client_with_repo
    content_type = headers.pop('Content-Type')

    file_name = '{0}'.format(uuid.uuid4().hex)
    response = svc_client.post(
        '/cache.files_upload',
        data=dict(file=(io.BytesIO(b'this is a test'), file_name), ),
        query_string={'override_existing': True},
        headers=headers
    )

    assert response
    assert 200 == response.status_code
    assert_rpc_response(response)

    assert 1 == len(response.json['result']['files'])
    file_id = response.json['result']['files'][0]['file_id']
    assert isinstance(uuid.UUID(file_id), uuid.UUID)

    payload = {
        'project_id': project_id,
        'short_name': 'mydata',
        'files': [{
            'file_id': file_id
        }, ],
    }
    headers['Content-Type'] = content_type

    response = svc_client.post(
        '/datasets.add',
        data=json.dumps(payload),
        headers=headers,
    )

    assert response
    assert_rpc_response(response)

    assert {'short_name', 'files',
            'project_id'} == set(response.json['result'].keys())
    assert file_id == response.json['result']['files'][0]['file_id']

    params = {
        'project_id': project_id,
        'short_name': 'mydata',
    }

    response = svc_client.get(
        '/datasets.files_list',
        query_string=params,
        headers=headers,
    )

    assert response
    assert_rpc_response(response)

    assert {'short_name', 'files'} == set(response.json['result'].keys())

    assert params['short_name'] == response.json['result']['short_name']
    assert file_name in [
        file['name'] for file in response.json['result']['files']
    ]


@pytest.mark.service
@pytest.mark.integration
@flaky(max_runs=30, min_passes=1)
def test_add_with_unpacked_archive(datapack_zip, svc_client_with_repo):
    """Upload archive and add it to a dataset."""
    svc_client, headers, project_id, _ = svc_client_with_repo
    content_type = headers.pop('Content-Type')

    response = svc_client.post(
        '/cache.files_upload',
        data=dict(
            file=(io.BytesIO(datapack_zip.read_bytes()), datapack_zip.name),
        ),
        query_string={
            'unpack_archive': True,
            'override_existing': True,
        },
        headers=headers
    )

    assert response
    assert_rpc_response(response)

    assert 200 == response.status_code
    assert response.json['result']['files']

    mm = {}
    for file_ in response.json['result']['files']:
        assert not file_['is_archive']
        assert not file_['unpack_archive']

        file_id = file_['file_id']
        assert file_id
        mm[file_['file_name']] = file_

    file_ = mm['file2']
    payload = {
        'project_id': project_id,
        'short_name': '{0}'.format(uuid.uuid4().hex),
    }

    headers['Content-Type'] = content_type
    response = svc_client.post(
        '/datasets.create',
        data=json.dumps(payload),
        headers=headers,
    )

    assert response
    assert_rpc_response(response)

    assert {'short_name'} == set(response.json['result'].keys())
    assert payload['short_name'] == response.json['result']['short_name']

    payload = {
        'project_id': project_id,
        'short_name': payload['short_name'],
        'files': [{
            'file_id': file_['file_id']
        }, ]
    }

    response = svc_client.post(
        '/datasets.add',
        data=json.dumps(payload),
        headers=headers,
    )

    assert response
    assert_rpc_response(response)

    assert {'short_name', 'files',
            'project_id'} == set(response.json['result'].keys())
    assert file_['file_id'] == response.json['result']['files'][0]['file_id']

    params = {
        'project_id': project_id,
        'short_name': payload['short_name'],
    }

    response = svc_client.get(
        '/datasets.files_list',
        query_string=params,
        headers=headers,
    )

    assert response
    assert_rpc_response(response)

    assert {'short_name', 'files'} == set(response.json['result'].keys())

    assert params['short_name'] == response.json['result']['short_name']
    assert file_['file_name'] in [
        file['name'] for file in response.json['result']['files']
    ]


@pytest.mark.service
@pytest.mark.integration
@flaky(max_runs=30, min_passes=1)
def test_add_with_unpacked_archive_all(datapack_zip, svc_client_with_repo):
    """Upload archive and add its contents to a dataset."""
    svc_client, headers, project_id, _ = svc_client_with_repo
    content_type = headers.pop('Content-Type')

    response = svc_client.post(
        '/cache.files_upload',
        data=dict(
            file=(io.BytesIO(datapack_zip.read_bytes()), datapack_zip.name),
        ),
        query_string={
            'unpack_archive': True,
            'override_existing': True,
        },
        headers=headers
    )

    assert response

    assert_rpc_response(response)
    assert 200 == response.status_code
    assert response.json['result']['files']

    mm = {}
    for file_ in response.json['result']['files']:
        assert not file_['is_archive']
        assert not file_['unpack_archive']

        file_id = file_['file_id']
        assert file_id
        mm[file_['file_name']] = file_

    file_ = mm['file2']

    files = [{
        'file_id': file_['file_id']
    } for file_ in response.json['result']['files']]

    payload = {
        'project_id': project_id,
        'short_name': '{0}'.format(uuid.uuid4().hex),
    }

    headers['Content-Type'] = content_type
    response = svc_client.post(
        '/datasets.create',
        data=json.dumps(payload),
        headers=headers,
    )

    assert response
    assert_rpc_response(response)

    assert {'short_name'} == set(response.json['result'].keys())
    assert payload['short_name'] == response.json['result']['short_name']

    payload = {
        'project_id': project_id,
        'short_name': payload['short_name'],
        'files': files,
    }

    response = svc_client.post(
        '/datasets.add',
        data=json.dumps(payload),
        headers=headers,
    )

    assert response
    assert_rpc_response(response)

    assert {'short_name', 'files',
            'project_id'} == set(response.json['result'].keys())
    assert files == response.json['result']['files']

    params = {
        'project_id': project_id,
        'short_name': payload['short_name'],
    }

    response = svc_client.get(
        '/datasets.files_list',
        query_string=params,
        headers=headers,
    )

    assert response
    assert_rpc_response(response)

    assert {'short_name', 'files'} == set(response.json['result'].keys())

    assert params['short_name'] == response.json['result']['short_name']
    assert file_['file_name'] in [
        file['name'] for file in response.json['result']['files']
    ]


@pytest.mark.service
@pytest.mark.integration
@flaky(max_runs=10, min_passes=1)
def test_add_existing_file(svc_client_with_repo):
    """Upload archive and add it to a dataset."""
    svc_client, headers, project_id, _ = svc_client_with_repo
    payload = {
        'project_id': project_id,
        'short_name': '{0}'.format(uuid.uuid4().hex),
    }

    response = svc_client.post(
        '/datasets.create',
        data=json.dumps(payload),
        headers=headers,
    )
    assert response
    assert_rpc_response(response)

    assert {'short_name'} == set(response.json['result'].keys())
    assert payload['short_name'] == response.json['result']['short_name']

    files = [{'file_path': 'README.md'}]
    payload = {
        'project_id': project_id,
        'short_name': payload['short_name'],
        'files': files,
    }

    response = svc_client.post(
        '/datasets.add',
        data=json.dumps(payload),
        headers=headers,
    )

    assert response
    assert_rpc_response(response)

    assert {'short_name', 'files',
            'project_id'} == set(response.json['result'].keys())

    assert files == response.json['result']['files']


@pytest.mark.parametrize(
    'doi',
    [
        '10.5281/zenodo.3239980', '10.7910/DVN/TJCLKP'
        # TODO: add http uri
    ]
)
@pytest.mark.integration
@pytest.mark.service
@flaky(max_runs=30, min_passes=1)
def test_import_dataset_job_enqueue(
    doi, svc_client_cache, project, mock_redis
):
    """Test import a dataset."""
    client, headers, cache = svc_client_cache
    user = cache.ensure_user({'user_id': 'user'})

    project_meta = {
        'project_id': uuid.uuid4().hex,
        'name': Path(project).name,
        'fullname': 'full project name',
        'email': 'my@email.com',
        'owner': 'me',
        'token': 'awesome token',
        'git_url': 'git@gitlab.com'
    }

    project_obj = cache.make_project(user, project_meta)

    dest = project_obj.abs_path
    os.makedirs(dest.parent, exist_ok=True)
    if not (project / dest).exists():
        shutil.copytree(project, dest)

    response = client.post(
        '/datasets.import',
        data=json.dumps({
            'project_id': project_meta['project_id'],
            'dataset_uri': doi,
        }),
        headers=headers,
    )

    assert_rpc_response(response)
    assert {
        'created_at',
        'job_id',
    } == set(response.json['result'])

    user_job = cache.get_job(user, response.json['result']['job_id'])
    assert response.json['result']['job_id'] == user_job.job_id

    response = client.get('/jobs', headers=headers)
    assert_rpc_response(response)
    assert response.json['result']['jobs']

    assert user_job.job_id in [
        job['job_id'] for job in response.json['result']['jobs']
    ]


@pytest.mark.parametrize(
    'url', [
        'https://gist.github.com/jsam/d957f306ed0fe4ff018e902df6a1c8e3',
    ]
)
@pytest.mark.integration
@pytest.mark.service
@flaky(max_runs=30, min_passes=1)
def test_dataset_add_remote(
    url, svc_client_cache, project_metadata, mock_redis
):
    """Test import a dataset."""
    project, project_meta = project_metadata
    client, headers, cache = svc_client_cache

    user = cache.ensure_user({'user_id': 'user'})
    project_obj = cache.make_project(user, project_meta)

    dest = project_obj.abs_path
    os.makedirs(dest.parent, exist_ok=True)
    if not (project / dest).exists():
        shutil.copytree(project, dest)

    payload = make_dataset_add_payload(project_meta['project_id'], [url])
    response = client.post(
        '/datasets.add',
        data=json.dumps(payload),
        headers=headers,
    )

    assert_rpc_response(response)
    assert {'files', 'short_name',
            'project_id'} == set(response.json['result'])
    job_id = response.json['result']['files'][0]['job_id']

    user_job = cache.get_job(user, job_id)
    assert job_id == user_job.job_id

    response = client.get('/jobs', headers=headers)
    assert_rpc_response(response)
    assert response.json['result']['jobs']

    assert user_job.job_id in [
        job['job_id'] for job in response.json['result']['jobs']
    ]


@pytest.mark.integration
@pytest.mark.service
@flaky(max_runs=30, min_passes=1)
def test_dataset_add_multiple_remote(
    svc_client_cache, project_metadata, mock_redis
):
    """Test dataset add multiple remote files."""
    project, project_meta = project_metadata
    url_gist = 'https://gist.github.com/jsam/d957f306ed0fe4ff018e902df6a1c8e3'
    url_dbox = 'https://www.dropbox.com/s/qcpts6fc81x6j4f/addme?dl=0'

    client, headers, cache = svc_client_cache
    user = cache.ensure_user({'user_id': 'user'})
    project_obj = cache.make_project(user, project_meta)

    dest = project_obj.abs_path
    os.makedirs(dest.parent, exist_ok=True)
    if not (project / dest).exists():
        shutil.copytree(project, dest)

    payload = make_dataset_add_payload(
        project_meta['project_id'], [url_gist, url_dbox]
    )
    response = client.post(
        '/datasets.add',
        data=json.dumps(payload),
        headers=headers,
    )

    assert_rpc_response(response)
    assert {'files', 'short_name',
            'project_id'} == set(response.json['result'])

    for file in response.json['result']['files']:
        job_id = file['job_id']

        user_job = cache.get_job(user, job_id)
        assert job_id == user_job.job_id

        response = client.get('/jobs', headers=headers)
        assert_rpc_response(response)
        assert response.json['result']['jobs']

        assert user_job.job_id in [
            job['job_id'] for job in response.json['result']['jobs']
        ]


@pytest.mark.service
@pytest.mark.integration
@flaky(max_runs=10, min_passes=1)
def test_add_remote_and_local_file(svc_client_with_repo):
    """Test dataset add remote and local files."""
    svc_client, headers, project_id, _ = svc_client_with_repo

    payload = make_dataset_add_payload(
        project_id,
        [('file_path', 'README.md'),
         'https://gist.github.com/jsam/d957f306ed0fe4ff018e902df6a1c8e3'],
    )
    response = svc_client.post(
        '/datasets.add',
        data=json.dumps(payload),
        headers=headers,
    )

    assert response
    assert_rpc_response(response)

    assert {'short_name', 'files',
            'project_id'} == set(response.json['result'].keys())

    for pair in zip(response.json['result']['files'], payload['files']):
        if 'job_id' in pair[0]:
            assert pair[0].pop('job_id')

        assert set(pair[0].values()) == set(pair[1].values())


@pytest.mark.service
@pytest.mark.integration
@flaky(max_runs=10, min_passes=1)
def test_edit_datasets_view(svc_client_with_repo):
    """Test editing dataset metadata."""
    svc_client, headers, project_id, _ = svc_client_with_repo
    short_name = '{0}'.format(uuid.uuid4().hex)

    payload = {
        'project_id': project_id,
        'short_name': short_name,
    }

    response = svc_client.post(
        '/datasets.create',
        data=json.dumps(payload),
        headers=headers,
    )

    assert response

    assert_rpc_response(response)
    assert {'short_name'} == set(response.json['result'].keys())
    assert payload['short_name'] == response.json['result']['short_name']

    params_list = {
        'project_id': project_id,
    }

    response = svc_client.get(
        '/datasets.list',
        query_string=params_list,
        headers=headers,
    )

    assert response
    assert_rpc_response(response)

    edit_payload = {
        'project_id': project_id,
        'short_name': short_name,
        'title': 'my new title',
    }
    response = svc_client.post(
        '/datasets.edit', data=json.dumps(edit_payload), headers=headers
    )

    assert response
    assert_rpc_response(response)

    assert {'warnings', 'edited'} == set(response.json['result'])
    assert {'title': 'my new title'} == response.json['result']['edited']



# -*- coding: utf-8 -*-
#
# Copyright 2019-2020 - Swiss Data Science Center (SDSC)
# A partnership between École Polytechnique Fédérale de Lausanne (EPFL) and
# Eidgenössische Technische Hochschule Zürich (ETHZ).
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Repository tests."""

import tempfile
from pathlib import Path


def test_init_repository(local_client):
    """Test initializing an empty repository."""
    local_client.init_repository()
    assert (local_client.path / '.git').exists()
    assert (local_client.path / '.git' / 'HEAD').exists()
    assert not (local_client.path / '.renku').exists()


def test_import_from_template(local_client):
    """Test importing data from template."""
    output_file = 'metadata.yml'
    local_client.init_repository()
    with tempfile.TemporaryDirectory() as tempdir:
        template_path = Path(tempdir)
        fake_template_file = template_path / output_file
        with fake_template_file.open('w') as dest:
            dest.writelines([
                'name: {{ name }}', 'description: {{ description }}',
                'created: {{ date_created }}', 'updated: {{ date_updated }}'
            ])
            metadata = {
                'name': 'name',
                'description': 'description',
                'date_created': 'now',
                'date_updated': 'now',
            }
        local_client.import_from_template(template_path, metadata)
        compiled_file = local_client.path / output_file
        compiled_content = compiled_file.read_text()
        expected_content = (
            'name: name'
            'description: description'
            'created: now'
            'updated: now'
        )
        assert expected_content == compiled_content



# -*- coding: utf-8 -*-
#
# Copyright 2018-2020 - Swiss Data Science Center (SDSC)
# A partnership between École Polytechnique Fédérale de Lausanne (EPFL) and
# Eidgenössische Technische Hochschule Zürich (ETHZ).
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Test agents."""
import pytest

from renku.core.models.provenance.agents import Person


@pytest.mark.parametrize(
    'value,has_name,has_email,has_affiliation', [
        ('John Doe<john.doe@mail.ch>[Some Affiliation]', True, True, True),
        (
            '  John Doe  <  john.doe@mail.ch  >  [  Some Affiliation  ]', True,
            True, True
        ),
        ('  John Doe  <  john.doe@mail.ch  >  [  ]', True, True, False),
        ('  John Doe  <  john.doe@mail.ch  >  ', True, True, False),
        (
            '  John Doe  <  john.doe@mail.ch  >  Some Affiliation ', True,
            True, False
        ),
        (
            '  <  john.doe@mail.ch  > [  Some Affiliation  ] ', False, True,
            True
        ),
        ('  <> [  Some Affiliation  ] ', False, False, True),
        ('  [  Some Affiliation  ] ', False, False, True),
        ('  <  john.doe@mail.ch  > [  ] ', False, True, False),
    ]
)
def test_construct_person(value, has_name, has_email, has_affiliation):
    """Test construct person from string."""
    p = Person.from_string(value)

    if has_name:
        assert 'John Doe' == p.name
    else:
        assert '' == p.name
    if has_email:
        assert 'john.doe@mail.ch' == p.email
    else:
        assert p.email is None
    if has_affiliation:
        assert 'Some Affiliation' == p.affiliation
    else:
        assert p.affiliation is None



# -*- coding: utf-8 -*-
#
# Copyright 2017-2020 - Swiss Data Science Center (SDSC)
# A partnership between École Polytechnique Fédérale de Lausanne (EPFL) and
# Eidgenössische Technische Hochschule Zürich (ETHZ).
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Python SDK and CLI for the Renku platform."""

import os
import shutil
from pathlib import Path
from tempfile import TemporaryDirectory

from setuptools import Command, find_packages, setup
from setuptools.command.bdist_egg import bdist_egg as _bdist_egg
from setuptools.command.build_ext import build_ext as _build_ext
from setuptools.command.build_py import build_py as _build_py
from setuptools.command.develop import develop as _develop

URL = 'https://github.com/SwissDataScienceCenter/renku-project-template'
REFERENCE = '0.1.10'


class DownloadTemplates(Command):
    description = 'Download renku repository templates'

    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        from renku.core.commands.init import fetch_template, \
            read_template_manifest

        with TemporaryDirectory() as tempdir:
            # download and extract template data
            temppath = Path(tempdir)
            print('downloading Renku templates...')
            fetch_template(URL, REFERENCE, temppath)
            read_template_manifest(temppath, checkout=True)

            # copy templates
            current_path = Path.cwd()
            template_path = current_path / 'renku' / 'templates'
            if template_path.exists():
                shutil.rmtree(str(template_path))
            shutil.copytree(
                str(temppath),
                str(template_path),
                ignore=shutil.ignore_patterns('.git')
            )


class bdist_egg(_bdist_egg):
    def run(self):
        self.run_command('DownloadTemplates')
        _bdist_egg.run(self)


class build_ext(_build_ext):
    def run(self):
        self.run_command('DownloadTemplates')
        _build_ext.run(self)


class build_py(_build_py):
    def run(self):
        self.run_command('DownloadTemplates')
        _build_py.run(self)


class develop(_develop):
    def run(self):
        self.run_command('DownloadTemplates')
        _develop.run(self)


readme = open('README.rst').read()
history = open('CHANGES.rst').read()

tests_require = [
    'check-manifest>=0.37',
    'coverage>=4.5.3',
    'fakeredis==1.4.1',
    'flake8>=3.5,<3.8',
    'flaky==3.6.1',
    'freezegun>=0.3.12',
    'isort==4.3.21',
    'six>=1.13.0',
    'pydocstyle>=3.0.0',
    'pytest-cache>=1.0',
    'pytest-cov>=2.5.1',
    'pytest-flake8>=1.0.4',
    'pytest-pep8>=1.0.6',
    'pytest-yapf>=0.1.1',
    'pytest>=4.0.0',
    'responses>=0.7.0',
    'unify>=0.4',
    'yapf==0.27.0',
]

extras_require = {
    'docs': [
        'Jinja2>=2.10.1',
        'Sphinx>=1.6.3',
        'renku-sphinx-theme>=0.1.0',
    ],
    'runner': ['cwlref-runner>=1.0', ],
    'notebook': [],
    'sentry': ['sentry-sdk>=0.7.4', ],
    'tests': tests_require,
}

setup_requires = [
    'pytest-runner>=2.6.2',
    'setuptools_scm>=3.1.0',
]

extras_require['all'] = list(setup_requires)
for name, reqs in extras_require.items():
    if name.startswith(':'):
        continue
    extras_require['all'].extend(reqs)

extras_require['nodocs'] = list(setup_requires)
for name, reqs in extras_require.items():
    if name.startswith(':') or name == 'docs':
        continue
    extras_require['nodocs'].extend(reqs)

install_requires = [
    'appdirs>=1.4.3',
    'apispec==3.3.0',
    'attrs>=19.3.0',
    'click-completion>=0.5.0',
    'click>=7.0',
    'cryptography>=2.7',
    'cwltool==3.0.20200324120055',
    'environ_config>=18.2.0',
    'filelock>=3.0.0',
    'flask==1.1.2',
    'flask-apispec==0.8.7',
    'flask-swagger-ui==3.25.0',
    'gitdb2==3.0.1',
    'gitpython==3.1.0',
    'humanize==2.4.0',
    'jinja2>=2.10.3',
    'pathspec>=0.7.0',
    'patool>=1.12',
    'pluggy>=0.13.1',
    'psutil>=5.4.7',
    'pyasn1>=0.4.5',
    'PyYAML>=3.12',
    'pyld<=1.0.5',
    'pyOpenSSL>=19.0.0',
    'pyshacl>=0.11.3.post1',
    'python-dateutil>=2.6.1',
    'python-editor>=1.0.4',
    'redis==3.4.1',
    'rdflib==4.2.2',
    'rdflib-jsonld>=0.4.0,<0.5.0',
    'requests>=2.21.0',
    'rq==1.3.0',
    'rq-scheduler==0.9.1',
    'ndg-httpsclient>=0.5.1',
    'marshmallow==3.5.2',
    'idna==2.8',
    'setuptools_scm>=3.1.0',
    'tabulate>=0.7.7',
    'tqdm==4.45.0',
    'walrus>=0.8.0',
    'wcmatch>=6.0.0',
    'werkzeug>=0.15.5',
]

complete_setup_requires = list(set(install_requires + setup_requires))

packages = find_packages()

version_template = """\
# -*- coding: utf-8 -*-
#
# Copyright 2017-%d - Swiss Data Science Center (SDSC)
# A partnership between École Polytechnique Fédérale de Lausanne (EPFL) and
# Eidgenössische Technische Hochschule Zürich (ETHZ).
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
\"\"\"Version information for Renku.\"\"\"

__version__ = {version!r}


def _get_disribution_url():
    try:
        import pkg_resources
        d = pkg_resources.get_distribution('renku')
        metadata = d._get_metadata(d.PKG_INFO)
        home_page = [m for m in metadata if m.startswith('Home-page:')]
        return home_page[0].split(':', 1)[1].strip()
    except Exception:
        return 'N/A'


version_url = '{{}}/tree/{{}}'.format(_get_disribution_url(), 'v' + __version__)
"""

setup(
    name='renku',
    use_scm_version={
        'local_scheme': 'dirty-tag',
        'write_to': os.path.join('renku', 'version.py'),
        'write_to_template': version_template,
    },
    description=__doc__,
    long_description=readme + '\n\n' + history,
    long_description_content_type='text/x-rst',
    keywords='Renku CLI',
    license='Apache License 2.0',
    author='Swiss Data Science Center',
    author_email='contact@datascience.ch',
    url='https://github.com/swissdatasciencecenter/renku-python',
    project_urls={
        'Changelog': (
            'https://github.com/swissdatasciencecenter/renku-python'
            '/blob/master/CHANGES.rst'
        ),
        'Docs': 'https://renku-python.rtfd.io/',
    },
    packages=packages,
    zip_safe=False,
    include_package_data=True,
    platforms='any',
    entry_points={
        'console_scripts': ['renku=renku.cli:cli'],
    },
    extras_require=extras_require,
    install_requires=install_requires,
    setup_requires=complete_setup_requires,
    tests_require=tests_require,
    cmdclass={
        'bdist_egg': bdist_egg,
        'build_py': build_py,
        'build_ext': build_ext,
        'develop': develop,
        'DownloadTemplates': DownloadTemplates,
    },
    classifiers=[
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Development Status :: 4 - Beta',
    ],
)



# -*- coding: utf-8 -*-
#
# Copyright 2020 - Swiss Data Science Center (SDSC)
# A partnership between École Polytechnique Fédérale de Lausanne (EPFL) and
# Eidgenössische Technische Hochschule Zürich (ETHZ).
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Renku service cache serializers for jobs."""
from marshmallow import Schema, fields

from renku.service.serializers.rpc import JsonRPCResponse


class JobDetails(Schema):
    """Response job details."""

    created_at = fields.DateTime()
    updated_at = fields.DateTime()

    job_id = fields.String(required=True)

    state = fields.String()
    extras = fields.Dict()


class JobListResponse(Schema):
    """Response schema for job listing."""

    jobs = fields.List(fields.Nested(JobDetails), required=True)


class JobListResponseRPC(JsonRPCResponse):
    """RPC response schema for jobs listing."""

    result = fields.Nested(JobListResponse)


class JobDetailsResponseRPC(Schema):
    """Response schema for job details."""

    result = fields.Nested(JobDetails)
